<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * This is the ajax front controller class for the Hard Hat Framework
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson <rod@spellmansystems.com>
 * 
 */

class sys_ajax_controller {

    private $_request;
    private $_registry;

/**
 * This is the constructor for the ajax Controller
 * 
 * @return void
 * 
 */    
    public function __construct(){
        $this->_registry = sys_registry::getInstance();
        $this->_request = sys_request::getInstance();
        $this->_commands = sys_commands::getInstance();
    }

/**
 * This function will process the request for the ajax controller
 * 
 * It will ensure that the url is a valid ajax command, and then load the appropriate
 * ajax action class.  It will also enforce any requirement that the action be restricted
 * to people properly logged in or system administrators.
 * 
 * @return void
 * 
 */
    public function processRequest() {
        $login = $this->_registry->getEntry('core', 'login');
        $cmd = $this->_request->getCommand();
        if ($cmd == 'error') {
           return;   
        }
        while ($cmd != 'done') {
            if (substr($cmd, 0, 1) == '>') {
                echo $cmd;
                exit;
            }
            $ajax = $this->_request->getCommandBean();
            if ($ajax == null) {
                echo ">Invalid ajax command $cmd";
                exit;
            }
            $template = $ajax->getTemplate();
            $obj = $ajax->getActionClass();
            if ($obj != '' && class_exists($obj)) {
                $act = new $obj();
            } else {
                echo '>System Error - Undefined Ajax Action '.$obj;
                exit;
            }
            $admin = $login->getAdmin();
            $loggedIn = $login->getLoggedIn();
            $adminReq = $ajax->getAdminReq();
            $loggedInReq = $ajax->getLoggedInReq();
            if (($adminReq == 'Y') && ($admin != 'Y')) {
                echo ">Not Authorized";
                exit;
            }
            if (($ajax->getLoggedInReq() == 'Y') && ($login->getLoggedIn() != 'Y')) {
                echo ">Must be logged in for this function";
                exit;
            }
            if (($login->getUserStatus() != 'A') && ($login->getUserStatus() != 'W')) {
                echo ">UserId is Suspended or Inactive - Contact Customer Support";
                exit;
            }
            $feedback = '';
            $cmd = $act->perform($ajax, $feedback, $login, $cmd);
        }
        $this->_registry->unsetEntry('core', 'feedback');
        $this->_registry->unsetEntry('core', 'postVars');
        $this->_registry->unsetEntry('sys', 'getVars');
    }

}