<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The sys_commands Object is a singleton that contains data from the conmands.xml file.  
 * It is used to map urls to action classes.  
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_commands {

    private $_instance;
    private $_xml;


    static public function getInstance(){
        static $_instance;
        if (!isset($_instance)) {
            $_instance = new sys_commands();
        }
        return $_instance;
    }

    private function __construct() {
        $this->_xml = simplexml_load_file(CONFIG_PATH."/config/commands.xml");
    }

    public function getDefaultPath() {
        $path = $this->_xml->xpath("//navigation_actions/defaultPath");
        if (count($path) == 0) {
            $path = 'home';
        } else {
            $path = $path[0];
        }
        return (string)$path;
    }

    public function getAction($path) {
        $defaults = $this->_xml->xpath("//post_actions/defaults");
        $defaults = $defaults[0];
        $actions = array();
        while (count($actions) == 0 && strlen($path) > 0) {
        	$actions = $this->_xml->xpath("//post_actions/post_action[path=\"$path\"]");
        	if (count($actions) == 0) {
           		$path = substr($path, 0, strrpos($path, '/'));
        	}
        }
        if (count($actions) > 0) {
        	$action = $actions[0]; 
           	$bean = new sys_nav_bean();
           	foreach ($defaults as $key => $val) {
           	    $bean->set($key,(string)$val);
           	}
           	foreach ($action as $key => $val) {
           	    $bean->set($key,(string)$val);
           	}
           	return $bean;
        } else return false;
    }

    public function getNavigation($path) {
        $defaults = $this->_xml->xpath("//navigation_actions/defaults");
        $defaults = $defaults[0];
        $actions = array();
        while (count($actions) == 0 && strlen($path) > 0) {
           $actions = $this->_xml->xpath("//navigation_actions/navigation_action[path=\"$path\"]");
           if (count($actions) == 0) {
           	$path = substr($path, 0, strrpos($path, '/')); 
           }   
        }
        if (count($actions) > 0) {
           $action = $actions[0]; 
           $bean = new sys_nav_bean();
           foreach ($defaults as $key => $val) {
               $bean->set($key,(string)$val);
           }
           foreach ($action as $key => $val) {
               $bean->set($key,(string)$val);
           }
           return $bean;
        } else return false;
    }

    public function getAjaxAction($path) {
    	$defaults = $this->_xml->xpath("//ajax_actions/defaults");
    	$defaults = $defaults[0];
        $actions = array();
        while (count($actions) == 0 && strlen($path) > 0) {
    		$actions = $this->_xml->xpath("//ajax_actions/ajax_action[path=\"$path\"]");
           if (count($actions) == 0) {
	           	$path = substr($path, 0, strrpos($path, '/')); 
           }   
        }
       	if (count($actions) > 0) {
    		$action = $actions[0];
    		$bean = new sys_nav_bean();
    		foreach ($defaults as $key => $val) {
    			$bean->set($key,(string)$val);
    		}
    		foreach ($action as $key => $val) {
    			$bean->set($key,(string)$val);
    		}
    		return $bean;
    	} else return false;
    }

    public function getBatchAction($path) {
    	$defaults = $this->_xml->xpath("//batch_actions/defaults");
    	$defaults = $defaults[0];
    	$actions = $this->_xml->xpath("//batch_actions/batch_action[path=\"$path\"]");
    	if (count($actions) > 0) {
    		$action = $actions[0];
    		$bean = new sys_nav_bean();
    		foreach ($defaults as $key => $val) {
    			$bean->set($key,(string)$val);
    		}
    		foreach ($action as $key => $val) {
    			$bean->set($key,(string)$val);
    		}
    		return $bean;
    	} else return false;
    }
    
    
}