<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2015, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The database connector for pdo.  It uses database credentials stored in the config.ini file which
 * is accessed through the sys_config singleton object.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_connect_pdo_db {

    private $_host;
    private $_user;
    private $_database;
    private $_connection;

    public function __construct($action = 'connect') {
        $config = sys_config::getInstance();
        $host     = $config->getEntry('host');
        $database = $config->getEntry('database');
        $user     = $config->getEntry('user');
        $password = $config->getEntry('password');

        $this->_host = $host;
        $this->_user = $user;
        $this->_database = $database;
        try {
        $this->_connection = new PDO("mysql:host=$host;dbname=$database;charset=utf8", $user, $password, 
        		array(PDO::ATTR_EMULATE_PREPARES => true, 
                      PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
    	}
        catch(PDOException $e)
        {
            throw new Exception("could not connect to server for host=$host, user=$user, database=$database, ".$e->getMessage());
        }        
    }

    static public function getInstance($action = 'connect'){
        static $connect_instance;
        if (!isset($connect_instance)) {
            $connect_instance = new sys_connect_pdo_db($action);
        }
        return $connect_instance;
    }

    public function getConnection() {
        return $this->_connection;
    }

    public function getHost() {
        return $this->_host;
    }
    public function getUser() {
        return $this->_user;
    }
    public function getDatabase() {
        return $this->_database;
    }

}
