<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This module is used to stage and send email.  It acts as a sort of governer to ensure that large
 * volumes of email don't overload the server, and also as a mechanism for recording all outgoing
 * emails.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_email {

	protected $_config;
	private $_list;
	private $_subject;
	private $_message;
	private $_headers;
	private $_fromEmail;
	private $_email_dao;
	private $_user;

	public function __get($var) {
		$className = get_class($this);
		throw new Exception("Use accessor method get$var to access private date in class $className. Could be missing () on a method call");
	}

	public function __set($var, $val){
		$className = get_class($this);
		throw new Exception("Use mutator method set$var to modify private date in class $className. Could be missing () on a method call");
	}

	// ---------------------------------------------------- constructor
	public function __construct(){
		$registry = sys_registry::getInstance();
		$this->_email_dao = new sys_email_dao;
		$this->_config = sys_config::getInstance();
		$this->_subject = '';
		$this->_message = '';
		$this->_fromEmail = $this->_config->getEntry('fromEmail');
		$this->_headers .= "Content-type: text/html\r\n";
		$this->_user = $registry->getEntry('core', 'login');
		if ($this->_user == null) {
			$this->_user = new sys_user_bean();
		}
	}

	public function setList($s) {
		$this->_list = $s;
	}
	public function setSubject($s) {
		$this->_subject = $s;
	}
	public function setMessage($s) {
		$this->_message = $s;
	}
	public function setHeaders($s) {
		$this->_headers = $s;
	}
	public function setFromEmail($s) {
		$this->_fromEmail = $s;
	}
	public function setUser($s) {
		$this->_user = $s;
	}
	public function getList() {
		return $this->_list;
	}
	public function getSubject() {
		return $this->_subject;
	}
	public function getMessage() {
		return $this->_message;
	}
	public function getHeaders() {
		return $this->_headers;
	}
	public function getFromEmail() {
		return $this->_fromEmail;
	}
	public function getUser() {
		return $this->_user;
	}

	public function formatTemplate($template, sys_base_bean $vars) {
		$serverName = SERVER;
		$version = APP_VERSION;
		$smarty = new Smarty;
		$smarty->compile_check = true;
		$smarty->debugging = false;
		$smarty->setTemplateDir(SMARTY_TEMPLATE_DIR);
		$smarty->setCompileDir(SMARTY_COMPILE_DIR);
		$smarty->setCacheDir(SMARTY_CACHE_DIR);
		$smarty->setConfigDir(SMARTY_CONFIG_DIR);      
		$datetime = date('r');
		$config = $this->_config->getConfig();
		foreach ($config as $key => $val) {
			if ($key != 'password') {
				$smarty->assign($key, $val);
			}
		}
		$fields = $vars->getBeanFields();
		foreach ($fields as $key => $val) {
			$smarty->assign($key, $val);
		}
		$smarty->assign("datetime", $datetime);
		$smarty->assign("version", $version);
		$smarty->assign("serverName", $serverName);
		$message = $smarty->fetch($template);
		$this->setMessage($message);
		return $message;
	}

	public function addToQueue ($aList = null, $aSubject = null, $aMessage = null, $aFromEmail = null, $aHeaders = null) {
		if ($aList == null) {
			$list = $this->getList();
		} else {
			$list = $aList;
		}
		if ($aSubject == null) {
			$subject = $this->getSubject();
		} else {
			$subject = $aSubject;
		}
		if ($aMessage == null) {
			$message = $this->getMessage();
		} else {
			$message = $aMessage;
		}
		if ($aFromEmail == null) {
			$fromEmail = $this->getFromEmail();
		} else {
			$fromEmail = $aFromEmail;
		}
		if ($aHeaders == null) {
			$headers = $this->getHeaders();
		} else {
			$headers = $aHeaders;
		}
		if (!is_array($list)) {
			$temp = $list;
			$list = array();
			$list [] = $temp;
		}
		$list = array_unique($list);
		$user = $this->getUser();
		$maxEmail = $this->_config->getEntry(('maxEmail'));
		$emailAddress = '';
		foreach ($list as $toEmail) {
			$bean = new sys_email_bean();
			$emailAddress = $toEmail;
			$bean->setToEmail($toEmail);
			$bean->setSubject($subject);
			$bean->setMessage($message);
			$bean->setHeaders($headers);
			$bean->setFromEmail($fromEmail);
			$bean->setSent('N');
			$this->_email_dao->insertEmail($bean, $user);
		}
		$hasMail = $this->_config->getEntry('hasMail');
		if ( $hasMail == 'Y') {
			$this->processQueue($maxEmail);
			return;
		} else {
			if ($this->_config->getEntry('showMail') == 'Y') {
				echo '<table border="0">';
				echo "<tr><td>email:</td>";
				echo "<td>$toEmail</td></tr>";
				echo "<tr><td>subject:</td>";
				echo "<td>$subject</td></tr>";
				echo "</table>";
				echo "message:<br />";
				echo "$message";
				exit;
			} else {
				return;
			}
		}
	}

	public function processQueue ($nbrToSend) {
		$user = $this->getUser();
		$beans = $this->_email_dao->loadUnprocessed();
		foreach ($beans as $bean) {
			$this->sendEmail($bean->getToEmail(),
			$bean->getSubject(),
			$bean->getMessage(),
			$bean->getFromEmail(),
			$bean->getHeaders());
			$bean->setSent('Y');
			$bean->setSendDateTime(fmtDbDateTime(strtotime('now')));
			$this->_email_dao->updateEmail($bean,$user);
		}
	}

	public function sendEmail ($toEmail, $subject, $message, $fromEmail, $headers) {
		$headers .= "Date: ".date('r')." \r\nFrom: $fromEmail \n";
		mail($toEmail, $subject, $message, $headers);
	}

}
