<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 *
 * This module will validate data based on the edits specified on the edits table.
 * There are two main functions here.  1) create JQuery validation routines for the
 * jquery.validate plugin.  and 2) validate data on the server side.  Both functions
 * are driven from the same edit table so that validation between client and server
 * are kept in sync.
 *
 * This routine validates against fields passed in the setFields method.  To validate
 * against the Post variables, see the routine sys_validation.php
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 *
 */
class sys_field_validator {

	private $_fields;

	// ---------------------------------------------------- constructor
	protected function __construct(){
		$this->fields = array();
		$this->_config = sys_config::getInstance();
	}

	function setFields($s) {
		$this->_fields = $s;
	}

	function getFields() {
		return $this->_fields;
	}

	public function createJqValidation($formName)  {
		$fields = $this->getFields();
		$messages = '';
		$result = "
		$().ready(function() {
		$('#$formName').validate({
		rules: {\n";
		foreach ($fields as $field) {
			$htmlName = $field->getHtmlName();
			$question = $field->getFieldName();
			$rule = '        '.$htmlName." : {\n";
			$msg  = '        '.$htmlName." : {\n";
			$keep = false;
			$edits = explode(',',$field->getEdits());
			foreach ($edits as $edit) {
				$tokens = explode('=',$edit);
				if ($tokens[0] == 'required') {
					$rule .= "            required: true,\n";
					$msg  .= "            required: 'required',\n";
					$keep = true;
				}
				if ($tokens[0] == 'number') {
					$rule .= "            number: true,\n";
					$msg  .= "            number: 'must be numeric',\n";
					$keep = true;
				}
				if ($tokens[0] == 'numeric') {
					$rule .= "            number: true,\n";
					$msg  .= "            number: 'must be numeric',\n";
					$keep = true;
				}
				if ($tokens[0] == 'maxlen') {
					$maxlen = $tokens[1];
					$rule .= "            maxlength: $maxlen,\n";
					$msg  .= "            maxlength: 'must be $maxlen characters or less',\n";
					$keep = true;
				}
				if ($tokens[0] == 'minlen') {
					$minlen = $tokens[1];
					$rule .= "            minlength: $minlen,\n";
					$msg  .= "            minlength: 'must be at least $minlen characters',\n";
					$keep = true;
				}
				if ($tokens[0] == 'max') {
					$max = $tokens[1];
					$rule .= "            max: $max,\n";
					$msg  .= "            max: 'cannot be more than $max',\n";
					$keep = true;
				}
				if ($tokens[0] == 'min') {
					$min = $tokens[1];
					$rule .= "            min: $min,\n";
					$msg  .= "            min: 'must be at least $min',\n";
					$keep = true;
				}
				if ($tokens[0] == 'email') {
					$rule .= "            email: true,\n";
					$msg  .= "            email: 'must be a valid email address',\n";
					$keep = true;
				}
				if ($tokens[0] == 'date') {
					$rule .= "            date: true,\n";
					$msg  .= "            date: 'must be a valid date',\n";
					$keep = true;
				}
			}
			if ($keep) {
				$result .= $rule;
				$result .= "\n";
				$result .= "	       },\n";
				$messages .= $msg;
				$messages = substr($messages,0,-2)."\n";
				$messages .= "	       },\n";
				$messages = substr($messages,0,-2).",\n";
			}
		}
		$messages = substr($messages,0,-2);
		$result = substr($result,0,-2);
		$result .= "
	},
				messages: {\n";
		$result .= $messages;
		$result .= "
	}
	});
	});";
		return $result;
	}

	function ValidData($feedback)
	{
		$dataIsValid = true;
		$fields = $this->_fields;

		foreach ($fields as $field) {
			if (!$this->ValidField($field,$feedback)) {
				$dataIsValid = false;
			}
		}
		return $dataIsValid;
	}

	function ValidField($field,$feedback) {
		$dataIsValid = true;

		$edits = explode(',',$field->getEdits());
		foreach ($edits as $edit) {
			$tokens = explode('=',$edit);
			if ($tokens[0] == 'required') {
				$dataIsValid = $this->EditReq($field,$feedback);
			}
			if ($tokens[0] == 'maxlen') {
				$field->setMaxLength($tokens[1]);
				$dataIsValid = $this->EditMaxLen($field,$feedback);
			}
			if ($tokens[0] == 'numeric') {
				$dataIsValid = $this->EditNum($field,$feedback);
			}
			if ($tokens[0] == 'date') {
				$dataIsValid = $this->EditDate($field,$feedback);
			}
			if ($tokens[0] == 'list') {
				$dataIsValid = $this->EditList($field,$feedback);
			}
			if ($tokens[0] == 'email') {
				$dataIsValid = $this->EditEmail($field,$feedback);
			}
		}
		return $dataIsValid;
	}

	function EditReq($field, $feedback) {
		$dataIsValid = true;
		$value = $field->getFieldValue();
		if (!isset($value)) {
			$editText = $field->getFieldName();
			$editText .= " is required";
			$feedback->setErr($editText);
			$dataIsValid = false;
		} else {
			if (trim($value) == '')  {
				$editText = $field->getFieldName();
				$editText .= " is required";
				$feedback->setErr($editText);
				$dataIsValid = false;
			}
		}
		return $dataIsValid;
	}

	function EditMaxLen($field, $feedback)  {
		$dataIsValid = true;
		$value = $field->getFieldValue();
		if (trim($value) == '')
			return $dataIsValid;
		$lengthFactor = strlen(trim($value));
		$maxFactor = $field->getMaxLength();
		if ($lengthFactor > $maxFactor) {
			$editText = $field->getFieldName();
			$editText .= " too long (max $maxFactor characters)";
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}


	function EditNum($field, $feedback) {
		$dataIsValid = true;
		$value = $field->getFieldValue();
		if (trim($value) == '')
			return $dataIsValid;
		if (!is_numeric($value)) {
			$editText = $field->getFieldName();
			$editText .= " must be numeric";
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}

	function EditDate($field, $feedback) {
		$dataIsValid = true;
		$value = $field->getFieldValue();
		if (trim($value) == '')
			return $dataIsValid;
		if (is_numeric($value) && strlen($value) == 6) {
			$m = substr($var,0,2);
			$d = substr($var,2,2);
			$y = substr($var,4,2);
			if ($y > 30) {
				$y += 1900;
			} else {
				$y += 2000;
			}
		} elseif (is_numeric($value) && strlen($value) == 8) {
			$m = substr($value,0,2);
			$d = substr($value,2,2);
			$y = substr($value,4,4);
		} elseif (strpos($value,'/') != false) {
			$date_array = explode('/',$value);
			$m = $date_array[0];
			$d = $date_array[1];
			$y = $date_array[2];
			if (strlen($y) == 2) {
				if ($y > 30) {
					$y += 1900;
				} else {
					$y += 2000;
				}
			}
		} elseif (strpos($var,'-') != false) {
			$date_array = explode('-',$value);
			$y = $date_array[0];
			$m = $date_array[1];
			$d = $date_array[2];
		}
		if (!checkdate($m,$d,$y)) {
			$editText = $field->getFieldName();
			$editText .= " must be a valid date";
			$feedback->setErr($editText);
			$dataIsValid = false;
		} else {
			$var = mktime(0,0,0,$m,$d,$y);
			$var = date('Y-m-d',$var);
			$field->setFieldValue($var);
		}
		return $dataIsValid;
	}
	
	function EditList($field, $feedback) {
		$dataIsValid = true;
		$value = $field->getFieldValue();
		if (trim($value) == '')
			return $dataIsValid;
		$fl = explode(';', $field->getFieldList());
		$found = false;
		foreach ($fl as $f) {
			if ($value == $f) $found = true;
		}
		if (!$found) {
			$editText = $field->getFieldName();
			$editText .= " has an invalid value";
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}
	

	function EditEmail($field, $feedback) {
		$dataIsValid = true;
		$value = trim($field->getFieldValue());
		if ($value == '')  {
			return true;
		}
		//        if(!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $var)) {
		if (!preg_match("/^[[:alnum:]][a-z0-9_.-]*@[a-z0-9.-]+\.[a-z]{2,4}$/i", $value)) {
			$editText = $field->getFieldName();
			$editText .= " is not a valid email";
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}
	
	
	/*

	function EditYesNo($field, $feedback) {
	$dataIsValid = true;
	$value = $field->getFieldValue();
	if ($value == 'X')
		return $dataIsValid;
	$dataIsValid = true;
	if (($value != 'Y') && ($value != 'N')) {
	$editText = $field->getFieldName();
	$editText .= " has an invalid value ";
	$feedback->setErr($editText);
	$dataIsValid = false;
	}
	return $dataIsValid;
	}

	*/
}