<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2016, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The sys_images Object is a library of image utilities.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_images {
	public function resizeImageAndCrop($fromFilename, $toFilename, $finalWidth, $finalHeight) {
		$size = getimagesize($fromFilename);
		if (!$size) {
			throw new Exception("getimagesize failed: $fromFilename");
		}
		$oldWidth = $size[0];
		$oldHeight = $size[1];
		$type = $size[2];
		$img_jpg = 2;
		$img_gif = 1;
		$img_png = 3;
		switch ($type){
			case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
			break;
			case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
			break;
			case $img_png : $oldImage = imagecreatefrompng($fromFilename);
			break;
			default :
				throw new Exception('unrecognized image type:'.$type);
		}
		$originalAspect = $oldWidth / $oldHeight;
		$finalAspect = $finalWidth / $finalHeight;
	
		if ( $originalAspect >= $finalAspect )
		{
			$newHeight = $finalHeight;
			$newWidth = $oldWidth / ($oldHeight / $finalHeight);
		}
		else
		{
			$newWidth = $finalWidth;
			$newHeight = $oldHeight / ($oldWidth / $finalWidth);
		}
	
		$x = floor(($newWidth - $finalWidth) / 2);
		if ($x < 0) {
			$x = 0;
		}
		$y = floor(($newHeight - $finalHeight) / 2);
		if ($y < 0) {
			$y= 0;
		}
		$newImage = imagecreatetruecolor($newWidth, $newHeight);
	
		if (!imagecopyresampled($newImage, $oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $oldWidth, $oldHeight)) {
			throw new Exception("imagecopyresampled failed");
		}
	
		$finalImage = imagecreatetruecolor($finalWidth, $finalHeight);
	
		if (!imagecopy(
				$finalImage,
				$newImage,
				0, 0,
				$x, $y,
				$finalWidth,
				$finalHeight)) {
					throw new Exception('imagecopy failed');
				}
	
				switch ($type){
					case $img_jpg : if (!imagejpeg($finalImage,$toFilename,90)) {
						throw new Exception("imagejpeg failed");
						return false;
					}
					break;
					case $img_gif : if (!imagegif($finalImage,$toFilename)) {
						throw new Exception("imagegif failed");
						return false;
					}
					break;
					case $img_png : if (!imagepng($finalImage,$toFilename,1)) {
						throw new Exception("imagepng failed");
						return false;
					}
					break;
				}
	
				return true;
	}
	public function resizeImageToWidth($toFilename, $fromFilename, $newWidth) {
	
		$size = getimagesize($fromFilename);
		if (!$size) {
			throw new Exception("getimagesize failed: $fromFilename");
			return false;
		}
		$width = $size[0];
		$height = $size[1];
		$type = $size[2];
		$img_jpg = 2;
		$img_gif = 1;
		$img_png = 3;
		switch ($type){
			case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
			break;
			case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
			break;
			case $img_png : $oldImage = imagecreatefrompng($fromFilename);
			break;
			default :
				throw new Exception("unrecognized image type: $type");
				return false;
		}
		$percentage =  $newWidth /$width;
		$newHeight = round($height * $percentage);
		$newImage = imagecreatetruecolor($newWidth, $newHeight);
	
		$colors = html2rgb($color);
		if (!imagecopyresampled($newImage, $oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)) {
			throw new Exception("imagecopyresampled failed");
			return false;
		}
		switch ($type){
			case $img_jpg : if (!imagejpeg($newImage,$toFilename,90)) {
				throw new Exception("imagejpeg failed");
				return false;
			}
			break;
			case $img_gif : if (!imagegif($newImage,$toFilename)) {
				throw new Exception("imagegif failed");
				return false;
			}
			break;
			case $img_png : if (!imagepng($newImage,$toFilename,1)) {
				throw new Exception("imagepng failed");
				return false;
			}
			break;
		}
	
		return true;
	}
	public function copyAndResizeImage($toFilename, $fromFilename, $maxWidth, $maxHeight) {
		return copyAndResizeImageWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, 'fff');
	}
	
	public function copyAndResizeImageWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, $color) {
	
		$size = getimagesize($fromFilename);
		if (!$size) {
			throw new Exception("getimagesize failed: $fromFilename");
			return false;
		}
		$width = $size[0];
		$height = $size[1];
		$type = $size[2];
		$img_jpg = 2;
		$img_gif = 1;
		$img_png = 3;
		switch ($type){
			case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
			break;
			case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
			break;
			case $img_png : $oldImage = imagecreatefrompng($fromFilename);
			break;
			default :
				throw new Exception("unrecognized image type: $type");
				return false;
		}
		$widthPercent = 1;
		if ($width > $maxWidth) {
			$widthPercent =  $maxWidth /$width;
		}
		$heightPercent = 1;
		if ($height > $maxHeight) {
			$heightPercent = $maxHeight / $height;
		}
		if ($widthPercent < $heightPercent) {
			$percentage = $widthPercent;
		} else {
			$percentage = $heightPercent;
		}
		$newWidth = round($width * $percentage);
		$newHeight = round($height * $percentage);
		$newX = round(($maxWidth - $newWidth) / 2);
		$newY = round(($maxHeight - $newHeight) / 2);
		$newImage = imagecreatetruecolor($maxWidth, $maxHeight);
	
		$colors = html2rgb($color);
		$backgroundColor = imagecolorallocate($newImage,$colors[0],$colors[1],$colors[2]);
		if (!imagefilltoborder($newImage,0,0,$backgroundColor,$backgroundColor)) {
			throw new Exception("imagefilltoborder failed");
			return false;
		}
		//*/
		if (!imagecopyresampled($newImage, $oldImage, $newX, $newY, 0, 0, $newWidth, $newHeight, $width, $height)) {
			throw new Exception("imagecopyresampled failed");
			return false;
		}
		switch ($type){
			case $img_jpg : if (!imagejpeg($newImage,$toFilename,90)) {
				throw new Exception("imagejpeg failed");
				return false;
			}
			break;
			case $img_gif : if (!imagegif($newImage,$toFilename)) {
				throw new Exception("imagegif failed");
				return false;
			}
			break;
			case $img_png : if (!imagepng($newImage,$toFilename,1)) {
				throw new Exception("imagepng failed");
				return false;
			}
			break;
		}
	
		return true;
	}
	
	public function resizeImage($fromFilename, $maxWidth, $maxHeight, $color='fff') {
	
		$size = getimagesize($fromFilename);
		if (!$size) {
			throw new Exception("getimagesize failed: $fromFilename");
			return false;
		}
		$width = $size[0];
		$height = $size[1];
		$type = $size[2];
		$img_jpg = 2;
		$img_gif = 1;
		$img_png = 3;
		switch ($type){
			case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
			break;
			case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
			break;
			case $img_png : $oldImage = imagecreatefrompng($fromFilename);
			break;
			default :
				throw new Exception("unrecognized image type: $type");
				return false;
		}
		$widthPercent = 1;
		if ($width > $maxWidth) {
			$widthPercent =  $maxWidth /$width;
		}
		$heightPercent = 1;
		if ($height > $maxHeight) {
			$heightPercent = $maxHeight / $height;
		}
		if ($widthPercent < $heightPercent) {
			$percentage = $widthPercent;
		} else {
			$percentage = $heightPercent;
		}
		$newWidth = round($width * $percentage);
		$newHeight = round($height * $percentage);
		$newX = round(($maxWidth - $newWidth) / 2);
		$newY = round(($maxHeight - $newHeight) / 2);
		$newImage = imagecreatetruecolor($maxWidth, $maxHeight);
	
		$colors = html2rgb($color);
		$backgroundColor = imagecolorallocate($newImage,$colors[0],$colors[1],$colors[2]);
		if (!imagefilltoborder($newImage,0,0,$backgroundColor,$backgroundColor)) {
			throw new Exception("imagefilltoborder failed");
			return false;
		}
		//*/
		if (!imagecopyresampled($newImage, $oldImage, $newX, $newY, 0, 0, $newWidth, $newHeight, $width, $height)) {
			throw new Exception("imagecopyresampled failed");
			return false;
		}
		return $newImage;
	}
	
	public function html2rgb($color) {
		if ($color[0] == '#')
			$color = substr($color, 1);
	
		if (strlen($color) == 6)
			list($r, $g, $b) = array($color[0].$color[1],
					$color[2].$color[3],
					$color[4].$color[5]);
			elseif (strlen($color) == 3)
			list($r, $g, $b) = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
			else
				return false;
	
			$r = hexdec($r); $g = hexdec($g); $b = hexdec($b);
	
			return array($r, $g, $b);
	}
	

}