<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The Registry Object is a singleton that contains any data needed by the system
 * except for configuration information.  It is a wrapper for the session variable.
 * It uses its own built in namespaces to segragate entries, typically by module
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_registry {

   private $_registry;

   static public function getInstance(){
      static $registry_instance;
      if (!isset($registry_instance)) {
         $registry_instance = new sys_registry();
      }
      return $registry_instance;
   }

   private function __construct() {
      session_start();
      if (isset($_SESSION['registry'])) {
          $this->_registry = $_SESSION['registry'];
      } else {
          $this->_registry = array();
          $this->setEntry('_sys', 'loggedIn', 'N');
          $_SESSION['registry'] = $this->_registry;
      }     
   }
   
   public function getNameSpace ($nameSpace) {
      if (!key_exists($nameSpace, $this->_registry)) {
         return null;
      }
      return $this->_registry[$nameSpace];
   }
   public function getEntry ($nameSpace, $entry) {
      if (!isset($this->_registry)) {
         return null;
      }
      $ns = $this->getNameSpace($nameSpace);
      if ($ns == null) {
         return null;
      }
      if (!key_exists($entry, $ns)) {
         return null;
      }
      return $ns[$entry];
   }
   public function issetEntry ($nameSpace, $entry) {
      if (!isset($this->_registry)) {
         return 'N';
      }
      $ns = $this->getNameSpace($nameSpace);
      if ($ns == null) {
         return 'N';
      }
      if (!key_exists($entry, $ns)) {
         return 'N';
      }
      if ($ns[$entry] == null) return 'N';
      return 'Y';
   }
   public function setEntry($nameSpace, $entry, $value) {
      $this->_registry[$nameSpace][$entry] = $value;
      $_SESSION['registry'] = $this->_registry;
      return;
   }
   public function initEntry($nameSpace, $entry, $value) {
      $tmp = $this->getEntry($nameSpace,$entry);
      if (isset($tmp)){
         return $tmp;
      }
      $this->_registry[$nameSpace][$entry] = $value;
      $_SESSION['registry'] = $this->_registry;
      return $value;
   }
   public function unsetEntry($nameSpace, $entry) {
      unset($this->_registry[$nameSpace][$entry]);
      $_SESSION['registry'] = $this->_registry;
      return;
   }
   
   public function loginSession($userId) {
      $this->setEntry('_sys', 'loggedIn', 'Y');
      return;
   }

   public function getLoggedIn() {
      return $this->getEntry('_sys', 'loggedIn');
   }

   public function logoutSession() {
      $_SESSION = array();
      $this->setEntry('_sys', 'loggedIn', 'N');
   }
   
   public function isSystemInitialized() {
       // to be considered initialized the database must be present
       // as well as all required tables
        try {
          $connect = sys_connect_db::getInstance();
        }
        catch(Exception $e)  {
          return false;
        } 
        try {
          $query  = "SHOW COLUMNS FROM user";
          $conn = $connect->getConnection();
          $result = mysql_query($query,$conn);
          if (!$result) return false;
        }
        catch(Exception $e) {
            return false;
        }
        return true;
   }

}