<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The Request Object is a singleton that contains data from the http request
 * in an easy to use format.  It parses the URL to return the component parts
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_request {

    private $_requestType;
    private $_ajax;
    private $_rawUrl;
    private $_command;
    private $_postVars;
    private $_urlFields;
    private $_commandBean;
    private $_parms;
    private $_message;


    static public function getInstance(){
        static $request_instance;
        if (!isset($request_instance)) {
            $request_instance = new sys_request();
        }
        return $request_instance;
    }

    private function __construct() {
        $this->_post_vars = array();
        $this->_urlFields = array();
        $this->_args = array();
        
        $this->_requestType = strtolower($_SERVER['REQUEST_METHOD']);
        $url = $_SERVER['REQUEST_URI'];
        $this->parseUrl($url);
    }
  
    private function getAction($url) {
        $commands = sys_commands::getInstance();
    	if ($this->isAjax()) {
    		$bean = $commands->getAjaxAction($url);
    	} elseif ($this->isPost()) {
    		$bean = $commands->getAction($url);
    	} else {
    		$bean = $commands->getNavigation($url);
    	}
    	return $bean;
    }
    
    public function parseUrl($url) {
        $registry = sys_registry::getInstance();
        $commands = sys_commands::getInstance();
        $url = ltrim($url, '/');
        $parts = explode("/", $url);
        $this->_ajax = $parts[0] == 'ajax' ? true : false;
        $this->_rawUrl = $url;
        $parts = explode('?', $url);
        $path = $parts[0];
        if (count($parts)>1) {
            $args = $parts[1];
        } else {
            $args = '';
        }
        $parts = explode('/', $path);
        $cmdurl = '';
        foreach ($parts as $part) {
            if (strpos($part, '=') === false) {
                $cmdurl .= '/'.$part;
            }  elseif (strpos($part, '=') !== false) {
                $cmdParts = explode('=',$part);
                if (count($cmdParts) > 1){
                    $key = $cmdParts[0];
                    $val = $cmdParts[1];
                    $this->_urlFields [$key] = $val;
                }
            }
        }
        $cmdurl = ltrim($cmdurl, '/');
        if (($cmdurl == '') or ($cmdurl == '/')) {
            $cmdurl = $commands->getDefaultPath();
        }
        $this->_command = $cmdurl;
        
        $parms = array();
        $bean = $this->getAction($cmdurl);
        $tempUrl = $url;
        while ($bean and $bean->getPath() != $tempUrl and $tempUrl != '') {
        	$parms [] = substr($tempUrl,strrpos($tempUrl, '/')+1);
        	$tempUrl = substr($tempUrl,0,strrpos($tempUrl, '/'));
        }
        
        $this->_parms = array_reverse($parms);
        if ($this->isAjax()) {
            $bean = $commands->getAjaxAction($cmdurl);
            if ($bean == null) {
                echo "Invalid URL for AJAX transaction - path not found: $url";
                return 'error';
            }
        } else {
            if ($this->isPost()) {
                $bean = $commands->getAction($cmdurl);
                if ($bean == null) {
                    $this->_message = "Invalid URL for post transaction - path not found: $url";
                    $this->_commandBean = $commands->getNavigation('error');
                    $registry->setEntry('core', 'postVars', $this->getPostVars());
                    $this->_requestType = 'get';
                    $this->setCommand('error');
                    return false;
                }
            } else {
                $bean = $commands->getNavigation($cmdurl);
                if ($bean == null) {
                    $this->_message = "Invalid URL for get transaction - path not found: $url";
                    $this->_commandBean = $commands->getNavigation('error');
                    $this->setCommand('error');
                    return false;
                }

            }
        }
        $this->_commandBean = $bean;
        $getVars = $registry->getEntry('sys', 'getVars');
        if (is_array($getVars)){
            foreach ($getVars as $key => $val) {
                $this->_urlFields [$key] = $val;
            }
        }
        $getVars = $_GET;
        if (is_array($getVars)){
            foreach ($getVars as $key => $val) {
                $this->_urlFields [$key] = $val;
            }
        }
        foreach ($_POST as $key => $val){
            $this->_postVars [$key] = $val;
        }
        return true;
    }
    
    
    public function getUrlFields () {
        return $this->_urlFields;
    }
    public function getUrlField ($key) {
//        $key = strtolower($key);
        if (key_exists($key,$this->_urlFields)) {
            return $this->_urlFields[$key];
        } 
        return false;
    }
    public function hasUrlField ($key) {
//        $key = strtolower($key);
        return key_exists($key,$this->_urlFields);
    }
    public function getRawUrl () {
        return $this->_rawUrl;
    }
    
    public function getSegment($id) {
    	$n = $id -1;
    	$url = $this->getRawUrl();
    	$segments = explode('/', $url);
    	if (key_exists($n, $segments)) {
    		return $segments[$n];
    	} else {
    		return false;
    	}
    	
    }
    public function getModule () {
        return $this->_module;
    }
    public function getMessage () {
        return $this->_message;
    }
    public function getCommand () {
        return $this->_command;
    }
    public function getCommandBean () {
        return $this->_commandBean;
    }
    public function getParms () {
        return $this->_parms;
    }
    public function getPostVars () {
        return $this->_postVars;
    }
    public function dumpPostVars () {
        $a = array();
        if (is_array($this->_postVars))
    	foreach ($this->_postVars as $key => $val) {
        	$a [] = $key.'='.$val;
        }
        return implode ('/',$a);
    }
    public function getPostVar ($a, $subcmd = null) {
    	if (!is_array($this->_postVars)) {
    		return false;
        }
        if (!key_exists($a,$this->_postVars)) {
            return false;
        }
        if ($subcmd == null) {
            return $this->_postVars[$a];
        } else {
            return sys_base_bean::convert($this->_postVars[$a],$subcmd);
        }
    }
    public function issetPostVar ($a) {
        return isset($this->_postVars[$a]);
    }
    public function hasPostVar ($a) {
        if (!is_array($this->_postVars)) return false;
        if (count($this->_postVars) == 0) return false;
        return key_exists($a, $this->_postVars);
    }
    public function hasPostVars () {
        if (!is_array($this->_postVars)) return false;
        if (count($this->_postVars) == 0) return false;
        return true;
    }
    public function hasPostVarStr ($a) {
        return key_exists($a, $this->_postVars) ? 'Y' : 'N';
    }

    public function getRequestType () {
        return $this->_requestType;
    }
    public function isGet () {
        return $this->_requestType == 'get' ? true : false;
    }
    public function isPost () {
        return $this->_requestType == 'post' ? true : false;
    }
    public function isAjax () {
        return $this->_ajax;
    }

    public function setFullCommand($cmd) {
        $className = get_class($this);
        $cmdParts = explode('/', $cmd);
        if (count($cmdParts) != 2) {
            throw new Exception("invalid action command $cmd in class $className");
        }
        $this->_module = $cmdParts[0];
        $this->_command = $cmdParts[1];
        return;
    }

    public function setFields ($fields) {
        $this->_fields = $fields;
    }
    public function setPostVars ($p) {
        $this->_postVars = $p;
    }
    public function setPostVar ($key, $value) {
        $this->_postVars[$key] = $value;
    }
    public function setField ($key, $value) {
        return $this->_fields[$key] = $value;
    }
    public function setRawUrl ($url) {
        $this->_rawUrl = $url;
    }
    public function setModule ($module) {
        $this->_module = $module;
    }
    public function setMessage ($message) {
        $this->_message = $message;
    }
    public function setCommand ($cmd) {
        $this->_command = $cmd;
    }


}