<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This module contains various utility function used by the system.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */

function rrmdir($dir) {
	if (is_dir($dir)) {
		$objects = scandir($dir);
		foreach ($objects as $object) {
			if ($object != "." && $object != "..") {
				if (filetype($dir.DIRECTORY_SEPARATOR.$object) == "dir") {
					rrmdir($dir.DIRECTORY_SEPARATOR.$object);
				} else {
					unlink($dir.DIRECTORY_SEPARATOR.$object);
				}
			}
		}
		reset($objects);
		rmdir($dir);
	}
}

function var_dump_pre($mixed = null) {
	echo '<pre>';
	var_dump($mixed);
	echo '</pre>';
	return null;
}

if (!function_exists('str_getcsv')) {
 
function str_getcsv($input, $delimiter=',', $enclosure='"', $escape=null, $eol=null) {
  $temp=fopen("php://memory", "rw");
  fwrite($temp, $input);
  fseek($temp, 0);
  $r=fgetcsv($temp, 4096, $delimiter, $enclosure);
  fclose($temp);
  return $r;
}
 
}

function callStack($stacktrace) {
	$i = 1;
	$return = '';
	foreach($stacktrace as $node) {
		@$return .=  "$i. ".basename($node['file']) .":" .$node['function'] ."(" .$node['line'].")\n";
		$i++;
	}
	return $return;
}



function dump($value,$level=0)
{
	if ($level==-1)
	{
		$trans[' ']='&there4;';
		$trans["\t"]='&rArr;';
		$trans["\n"]='&para;;';
		$trans["\r"]='&lArr;';
		$trans["\0"]='&oplus;';
		return strtr(htmlspecialchars($value),$trans);
	}
	if ($level==0) echo '<pre>';
	$type= gettype($value);
	echo $type;
	if ($type=='string')
	{
		echo '('.strlen($value).')';
		$value= dump($value,-1);
	}
	elseif ($type=='boolean') $value= ($value?'true':'false');
	elseif ($type=='object')
	{
		$props= get_class_vars(get_class($value));
		echo '('.count($props).') <u>'.get_class($value).'</u>';
		foreach($props as $key=>$val)
		{
			echo "\n".str_repeat("\t",$level+1).$key.' => ';
			dump($value->$key,$level+1);
		}
		$value= '';
	}
	elseif ($type=='array')
	{
		echo '('.count($value).')';
		foreach($value as $key=>$val)
		{
			echo "\n".str_repeat("\t",$level+1).dump($key,-1).' => ';
			dump($val,$level+1);
		}
		$value= '';
	}
	echo " <b>$value</b>";
	if ($level==0) echo '</pre>';
}



function has_Mail(){
	//  if ($_SESSION['hasMail'] == 'Y') {
	//    return true;
	//  } else {
	return false;
	//  }
}
function resizeImageAndCrop($fromFilename, $toFilename, $finalWidth, $finalHeight, $feedback) {
    $size = getimagesize($fromFilename);
	if (!is_array($size)) {
			$feedback->setErr("getimagesize failed: $fromFilename");
			return false;
		//throw new Exception("getimagesize failed: $fromFilename");
	}
	$oldWidth = $size[0];
	$oldHeight = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$originalAspect = $oldWidth / $oldHeight;
	$finalAspect = $finalWidth / $finalHeight;
	
	if ( $originalAspect >= $finalAspect )
	{
		$newHeight = $finalHeight;
		$newWidth = $oldWidth / ($oldHeight / $finalHeight);
	}
	else
	{
		$newWidth = $finalWidth;
		$newHeight = $oldHeight / ($oldWidth / $finalWidth);
	}

	$x = floor(($newWidth - $finalWidth) / 2);
	if ($x < 0) {
		$x = 0;
	}
	$y = floor(($newHeight - $finalHeight) / 2);
	if ($y < 0) {
		$y= 0;
	}
	

/*
	echo "originalAspect=".$originalAspect."<br />";
	echo "finalAspect=".$finalAspect."<br />";
	echo "oldWidth=".$oldWidth."<br />";
	echo "oldHeight=".$oldHeight."<br />";
	echo "finalWidth=".$finalWidth."<br />";
	echo "finalHeight=".$finalHeight."<br />";
	echo "newWidth=".$newWidth."<br />";
	echo "newHeight=".$newHeight."<br />";
	echo "x=".$x."<br />";
	echo "y=".$y."<br />";
	exit;	
//*/	
	$newImage = imagecreatetruecolor($newWidth, $newHeight);
	if($type == $img_gif or $type == $img_png){
		imagecolortransparent($newImage, imagecolorallocatealpha($newImage, 0, 0, 0, 127));
		imagealphablending($newImage, false);
		imagesavealpha($newImage, true);
	}
	
	if (!imagecopyresampled($newImage, $oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $oldWidth, $oldHeight)) {
			$feedback->setErr("imagecopyresampled failed");
			return false;
		//throw new Exception("imagecopyresampled failed");
	}
	$finalImage = imagecreatetruecolor($finalWidth, $finalHeight);
	if($type == $img_gif or $type == $img_png){
		imagecolortransparent($finalImage, imagecolorallocatealpha($finalImage, 0, 0, 0, 127));
		imagealphablending($finalImage, false);
		imagesavealpha($finalImage, true);
	}	

	if (!imagecopy(
			$finalImage,
			$newImage, 
			0, 0, 
			$x, $y,
			$finalWidth, 
			$finalHeight)) {
				$feedback->setErr("imagecopy failed");
				return false;
//				throw new Exception('imagecopy failed');
			}
	switch ($type){
		case $img_jpg : if (!imagejpeg($finalImage,$toFilename,100)) {
			$feedback->setErr("imagejpeg failed");
			return false;
//			throw new Exception("imagejpeg failed");
		}
		break;
		case $img_gif : if (!imagegif($finalImage,$toFilename)) {
			$feedback->setErr("imagegif failed");
			return false;
			//throw new Exception("imagegif failed");
		}
		break;
		case $img_png : if (!imagepng($finalImage,$toFilename,1)) {
			$feedback->setErr("imagepng failed");
			return false;
			//throw new Exception("imagepng failed");
		}
		break;
	}
	
	return true;
}
function resizeImageToWidth($toFilename, $fromFilename, $newWidth, $feedback) {

	$size = getimagesize($fromFilename);
	if (!$size) {
		$feedback->setErr("getimagesize failed: $fromFilename");
		return false;
	}
	$width = $size[0];
	$height = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$percentage =  $newWidth /$width;
	$newHeight = round($height * $percentage);
	$newImage = imagecreatetruecolor($newWidth, $newHeight);

	if (!imagecopyresampled($newImage, $oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)) {
		$feedback->setErr("imagecopyresampled failed <br /> imagecopyresampled($newImage, $$oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)");
		return false;
	}
	switch ($type){
		case $img_jpg : if (!imagejpeg($newImage,$toFilename,100)) {
			$feedback->setErr("imagejpeg failed");
			return false;
		}
		break;
		case $img_gif : if (!imagegif($newImage,$toFilename)) {
			$feedback->setErr("imagegif failed");
			return false;
		}
		break;
		case $img_png : if (!imagepng($newImage,$toFilename,1)) {
			$feedback->setErr("imagepng failed");
			return false;
		}
		break;
	}

	return true;
}
function resizeImageToHeight($toFilename, $fromFilename, $newHeight, $feedback) {

	$size = getimagesize($fromFilename);
	if (!$size) {
		$feedback->setErr("getimagesize failed: $fromFilename");
		return false;
	}
	$width = $size[0];
	$height = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$percentage =  $newHeight /$height;
	$newWidth = round($width * $percentage);
	$newImage = imagecreatetruecolor($newWidth, $newHeight);

	if (!imagecopyresampled($newImage, $oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)) {
		$feedback->setErr("imagecopyresampled failed <br /> imagecopyresampled($newImage, $$oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)");
		return false;
	}
	switch ($type){
		case $img_jpg : if (!imagejpeg($newImage,$toFilename,100)) {
			$feedback->setErr("imagejpeg failed");
			return false;
		}
		break;
		case $img_gif : if (!imagegif($newImage,$toFilename)) {
			$feedback->setErr("imagegif failed");
			return false;
		}
		break;
		case $img_png : if (!imagepng($newImage,$toFilename,1)) {
			$feedback->setErr("imagepng failed");
			return false;
		}
		break;
	}

	return true;
}

function copyAndResizeImage($toFilename, $fromFilename, $maxWidth, $maxHeight, $feedback) {
      return copyAndResizeImageWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, 'fff', $feedback);
}	

function copyAndResizeImageWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, $color, $feedback) {

	$size = getimagesize($fromFilename);
	if (!$size) {
		$feedback->setErr("getimagesize failed: $fromFilename");
		return false;
	}
	$width = $size[0];
	$height = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$widthPercent = 1;
	if ($width > $maxWidth) {
		$widthPercent =  $maxWidth /$width;
	}
	$heightPercent = 1;
	if ($height > $maxHeight) {
		$heightPercent = $maxHeight / $height;
	}
	if ($widthPercent < $heightPercent) {
		$percentage = $widthPercent;
	} else {
		$percentage = $heightPercent;
	}
	$newWidth = round($width * $percentage);
	$newHeight = round($height * $percentage);
	$newX = round(($maxWidth - $newWidth) / 2);
	$newY = round(($maxHeight - $newHeight) / 2);
	$newImage = imagecreatetruecolor($maxWidth, $maxHeight);

	$colors = html2rgb($color);
	$backgroundColor = imagecolorallocate($newImage,$colors[0],$colors[1],$colors[2]);
	if (!imagefilltoborder($newImage,0,0,$backgroundColor,$backgroundColor)) {
		$feedback->setErr("imagefilltoborder failed");
		return false;
	}
	//*/
	if (!imagecopyresampled($newImage, $oldImage, $newX, $newY, 0, 0, $newWidth, $newHeight, $width, $height)) {
		$feedback->setErr("imagecopyresampled failed <br /> imagecopyresampled($newImage, $$oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)");
		return false;
	}
	switch ($type){
		case $img_jpg : if (!imagejpeg($newImage,$toFilename,100)) {
			$feedback->setErr("imagejpeg failed");
			return false;
		}
		break;
		case $img_gif : if (!imagegif($newImage,$toFilename)) {
			$feedback->setErr("imagegif failed");
			return false;
		}
		break;
		case $img_png : if (!imagepng($newImage,$toFilename,1)) {
			$feedback->setErr("imagepng failed");
			return false;
		}
		break;
	}

	return true;
}

function resizeImage($fromFilename, $maxWidth, $maxHeight, $color='fff', $feedback) {

	$size = getimagesize($fromFilename);
	if (!$size) {
		$feedback->setErr("getimagesize failed: $fromFilename");
		return false;
	}
	$width = $size[0];
	$height = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$widthPercent = 1;
	if ($width > $maxWidth) {
		$widthPercent =  $maxWidth /$width;
	}
	$heightPercent = 1;
	if ($height > $maxHeight) {
		$heightPercent = $maxHeight / $height;
	}
	if ($widthPercent < $heightPercent) {
		$percentage = $widthPercent;
	} else {
		$percentage = $heightPercent;
	}
	$newWidth = round($width * $percentage);
	$newHeight = round($height * $percentage);
	$newX = round(($maxWidth - $newWidth) / 2);
	$newY = round(($maxHeight - $newHeight) / 2);
	$newImage = imagecreatetruecolor($maxWidth, $maxHeight);

	$colors = html2rgb($color);
	$backgroundColor = imagecolorallocate($newImage,$colors[0],$colors[1],$colors[2]);
	if (!imagefilltoborder($newImage,0,0,$backgroundColor,$backgroundColor)) {
		$feedback->setErr("imagefilltoborder failed");
		return false;
	}
	//*/
	if (!imagecopyresampled($newImage, $oldImage, $newX, $newY, 0, 0, $newWidth, $newHeight, $width, $height)) {
		$feedback->setErr("imagecopyresampled failed <br /> imagecopyresampled($newImage, $$oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)");
		return false;
	}
	return $newImage;
}

function copyAndResizeJpeg($toFilename, $fromFilename, $maxWidth, $maxHeight, $feedback) {
	return copyAndResizeJpegWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, 'fff', $feedback);
}

function copyAndResizeJpegWBackground($toFilename, $fromFilename, $maxWidth, $maxHeight, $color, $feedback) {

	$size = getimagesize($fromFilename);
	if (!$size) {
		$feedback->setErr("getimagesize failed: $fromFilename");
		return false;
	}
	$width = $size[0];
	$height = $size[1];
	$type = $size[2];
	$img_jpg = 2;
	$img_gif = 1;
	$img_png = 3;
	switch ($type){
		case $img_jpg : $oldImage = imagecreatefromjpeg($fromFilename);
		break;
		case $img_gif : $oldImage = imagecreatefromgif($fromFilename);
		break;
		case $img_png : $oldImage = imagecreatefrompng($fromFilename);
		break;
		default :
			$feedback->setErr("unrecognized type: $type");
			return false;
	}
	$widthPercent = 1;
	if ($width > $maxWidth) {
		$widthPercent =  $maxWidth /$width;
	}
	$heightPercent = 1;
	if ($height > $maxHeight) {
		$heightPercent = $maxHeight / $height;
	}
	if ($widthPercent < $heightPercent) {
		$percentage = $widthPercent;
	} else {
		$percentage = $heightPercent;
	}
	$newWidth = round($width * $percentage);
	$newHeight = round($height * $percentage);
	$newX = round(($maxWidth - $newWidth) / 2);
	$newY = round(($maxHeight - $newHeight) / 2);
	$newImage = imagecreatetruecolor($maxWidth, $maxHeight);

	$colors = html2rgb($color);
	$backgroundColor = imagecolorallocate($newImage,$colors[0],$colors[1],$colors[2]);
	if (!imagefilltoborder($newImage,0,0,$backgroundColor,$backgroundColor)) {
		$feedback->setErr("imagefilltoborder failed");
		return false;
	}
	//*/
	if (!imagecopyresampled($newImage, $oldImage, $newX, $newY, 0, 0, $newWidth, $newHeight, $width, $height)) {
		$feedback->setErr("imagecopyresampled failed <br /> imagecopyresampled($newImage, $$oldImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height)");
		return false;
	}
   if (!imagejpeg($newImage,$toFilename,100)) {
        $feedback->setErr("imagejpeg failed");
        return false;
    }
    return true;
}

function html2rgb($color)
{
	if ($color[0] == '#')
		$color = substr($color, 1);

	if (strlen($color) == 6)
		list($r, $g, $b) = array($color[0].$color[1],
				$color[2].$color[3],
				$color[4].$color[5]);
	elseif (strlen($color) == 3)
	list($r, $g, $b) = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
	else
		return false;

	$r = hexdec($r); $g = hexdec($g); $b = hexdec($b);

	return array($r, $g, $b);
}

function makeURL($cmd, $request){
	//    $server = $_SESSION['server'];
	$cmdParts = explode('/', $cmd);
	if (count($cmdParts) != 2) {
		$fileName = __FILE__;
		throw new Exception("invalid command $cmd in $fileName");
	}
	$module  = $cmdParts[0];
	$cmdName = $cmdParts[1];
	$urlString = "./$module/$cmdName";
	//    $request = sys_request::getInstance();
	$urlFields = $request->getUrlFields();
	foreach ($urlFields as $key => $value) {
		$urlString .= "/$key/$value";
	}
	return $urlString;
}

function makeRelativeURL($cmd){
	if (substr($cmd,0,1) == '/') {
		$urlString = "$cmd";
	} else {
		$urlString = "/$cmd";
	}
	return $urlString;
}


function makeArray($t){
	$s = $t;
	$a = array();
	snip($s,':');
	$n = snip($s,':');
	for ($i=1;$i<=$n;$i++) {
		snip($s,"\"");
		$k = snip($s,"\"");
		snip($s,"\"");
		$v = snip($s,"\"");
		$a[$k] = $v;
	}
	return $a;
}

function formatDebug($type,$obj,$login,$feedback,$pageFrom,$submit,$cmd,$template) {
	$db = new debug_bean();
	$db->setType($type);
	$db->setAction($obj);
	$db->setLogin($login);
	$db->setUserName($login->getUserName());
	$db->setGroupName($login->getGroupName());
	$db->setFeedback($feedback);
	$db->setPageFrom($pageFrom);
	$db->setSubmit($submit);
	$db->setCmd($cmd);
	$db->setTemplate($template);
	return $db;
}

function fmtIntDefault($i,$d)
{
	if (!is_numeric($i)){
		return $d;
	}
	return $i;
}

function fmtInt($i)
{
	$i = trim($i);
	if (!is_numeric($i)){
		return 'null';
	}
	$r = (int)($i);
	return $r;
}

function fixInt($w,$n)
{
	$s = $n;
	$i = $w - strlen($n);
	for ($i=$w-strlen($n);$i>0;$i--){
		$s = "0$s";
	}
	return $s;
}

function fmtString($s)
{
	if (!isset($s)){
		return 'null';
	}
	$r = "'$s'";
	return $r;
}

function fmtDate($d)
{
	if (!isset($d)){
		return 'null';
	}
	$r = "'$d'";
	return $r;
}

function fmtDbDate($d)
{
	if (!isset($d)){
		return 'null';
	}
	if (!is_int($d)) {
		return 'null';
	}
	$r = date('c',$d);
	$r = substr($r,0,strlen($r)-7);
	$r = "'$r'";
	return $r;
}

function fmtDbDateTime($d)
{
	if (!isset($d)){
		return 'null';
	}
	$r = date('c',$d);
	$r = "'$r'";
	return $r;
}

function fmtCurrentDate()
{
	
	$r = date('Y-m-d H:i:s');
	$r = "'$r'";
	return $r;
}
function fmtCurrentDateShort()
{
	$r = date('c');
	$r = substr($r,0,10);
	return $r;
}

function format_currency($n)
{
	$s = '$';
	$s .= number_format($n,2);
	return $s;
}

function fmtDecimal($n)
{
	if (!isset($n)){
		return 'null';
	}
	return $n;
}


function snip(&$s,$sep) {
	$s_array = explode($sep,$s);
	$r = $s_array [0];
	$s = substr($s,strlen($r)+1,strlen($s));
	return $r;
}

function snipstr(&$s,$sep) {
	$s_array = explode($sep,$s);
	$r = $s_array [0];
	$s = substr($s,strlen($r)+strlen($sep),strlen($s));
	return $r;
}

function myStrToTime($timeStr) {
	$s = $timeStr;
	$h = snip($s,':');
	$m = substr($s,2);
	$amPm = substr($s,2,2);
	if (strtolower($amPm) == 'pm') {
		$h += 12;
	}
	return mktime((int)$h,(int)$m,0,0,0,0);
}

function strToDate($dt) {
	$m = 0;
	$d = 0;
	$y = 0;
	if (is_numeric($dt) && strlen($dt) == 6) {
		$m = substr($d,0,2);
		$d = substr($d,2,2);
		$y = substr($d,4,2);
		if ($y > 30) {
			$y += 1900;
		} else {
			$y += 2000;
		}
	} elseif (is_numeric($dt) && strlen($dt) == 8) {
		$m = substr($d,0,2);
		$d = substr($d,2,2);
		$y = substr($d,4,4);
	} elseif (strpos($dt,'/') != false) {
		$date_array = explode('/',$dt);
		$m = $date_array[0];
		$d = $date_array[1];
		$y = $date_array[2];
		if (strlen($y) == 2) {
			if ($y > 30) {
				$y += 1900;
			} else {
				$y += 2000;
			}
		}
	} elseif (strpos($dt,'-') != false) {
		$date_array = explode('-',$dt);
		$y = $date_array[0];
		$m = $date_array[1];
		$d = $date_array[2];
	}
	if (!checkdate($m,$d,$y)) {
		return null;
	} else {
		$var = mktime(0,0,0,$m,$d,$y);
	}
	return $var;
}

function getConfigEntry($entry) {
	$config = sys_config::getInstance();
	return $config->getEntry($entry);
}

function xWeekday($x, $w, $dt) {
	switch (strtolower($w)) {
		case "sunday"    : $w = 0; 
		break;
		case "monday"    : $w = 1;
		break;
		case "tuesday"   : $w = 2;
		break;
		case "wednesday" : $w = 3;
		break;
		case "thursday"  : $w = 4;
		break;
		case "friday" 	 : $w = 5;
		break;
		case "saturday"  : $w = 6;
	}
	$x = filter_var($x, FILTER_SANITIZE_NUMBER_INT);
	$m = date('n',$dt);
	$d = date('j',$dt);
	$y = date('y',$dt);
	$xCnt = 0;
	$lastDay = 0;
	$workDate = mktime(0,0,0,$m,1,$y);
	$nbrInMonth = date('t',$workDate);
	for ($i=1;$i<=$nbrInMonth;$i++) {
		if ($w == date('w',mktime(0,0,0,$m,$i,$y))) {
			$xCnt++;
			if ($xCnt == $x) {
				$d = $i;
				break;
			}
		}
	}
	return mktime(0,0,1,$m,$d,$y);
}


function findWhichDay($dt, $w) {
	$m = date('n',$dt);
	$d = date('j',$dt);
	$y = date('y',$dt);
	$xCnt = 0;
	$workDate = mktime(0,0,0,$m,1,$y);
	$nbrInMonth = date('t',$workDate);
	$i = 1;
	while ($workDate <= $dt) {
		if ($w == date('w',mktime(0,0,0,$m,$i,$y))) {
			$xCnt++;
		}
		$i++;
		$workDate = mktime(0,0,0,$m,$i,$y);
	}
	return $xCnt > 4 ? 'L' : $xCnt;
}


function get_random_password()
{
	$s = '';
	for ($i=0;$i<4;$i++){
		$c = chr(rand(65,90));
		$s = "$s$c";
		$c = chr(rand(48,57));
		$s = "$s$c";
	}
	$s = strtolower($s);
	$s = str_replace('0','x',$s);
	$s = str_replace('o','y',$s);
	$s = str_replace('l','4',$s);
	$s = str_replace('1','5',$s);
	return $s;
}

function get_random_string($length) {
	$r = md5( uniqid ( rand() ) );
	//   $length = -1 * $length ;
	//   echo $length;
	$r = substr($r,$length*-1);
	return $r;
}

function reverse_str($str) {
	$s = '';
	$i = strlen($str);
	while ($i > 0) {
		$s .= substr($str,$i,1);
		$i--;
	}
	return $s;
}

/*
 function formatMessage($feedback) {
 $msg = $feedback->getMsg();
 if ($msg == '') {
 return "";
 } else {
 return "<div id=\"message\">$msg";
 }
 }

 function addError($message,$type,$feedback){
 $err = new error_bean(null, date('m/d/Y H:i:s'), '',
 $message, '');
 $feedback->setErr($err);
 $feedback->setMsg($type);
 }

 function formatErrors($feedback) {
 $msg = $feedback->getMsg();
 $errors = $feedback->getErrors();
 $divStarted = false;
 $divEnded = false;
 if ($msg != '') {
 $divStarted = true;
 }
 if (count($errors) == 0) {
 if ($divStarted) {
 $errorList = "</div>";
 } else {
 $errorList = "";
 }
 } else {
 if (!$divStarted) {$errorList = '<div id="message">';}
 $errorList = '<ul>';
 foreach ($errors as $err) {
 $t = $err->getMessage();
 $errorList = "$errorList <li> $t </li>";
 }
 $errorList = "$errorList </ul></div>";
 }
 return $errorList;
 }
 //*/

function sendTheMail ($email, $subject, $message, $headers) {
	if (has_Mail()) {
		mail($email, $subject, $message, $headers);
		//         echo "email: $email\n\r";
		//         echo "subject: $subject\n\r";
		//         echo "message: $message\n\r";
		//         echo "headers: $headers\n\r";
	} else {
		if (isset($_SESSION['showMail'])) {
			//    if ($_SESSION['showMail'] == 'Y') {
			echo '<table border="1">';
			echo "<tr><td>email:</td>";
			echo "<td>$email</td></tr>";
			echo "<tr><td>subject:</td>";
			echo "<td>$subject</td></tr>";
			echo "</table>";
			echo "message:<br />";
			echo "$message";
			exit;
			//   }
		}
	}

}


//A suite of financial management functions including present and future
// value of an annuity and a loan payment calculation function

/* 

VARIABLES: 

$m  =  number  of  periods  per  year.    For  instance,  if  you  are  For   
instance,  if  you  are  making  payments  monthly  then  this  number  is  12. 
$n  =  total  number  of  periods.    For  instance,  if  you  are   
compounding  monthly  for  5  years,  this  number  would  be  60  (12  months   
x  5  years). 
$R  =  annual  interest  rate  (represented  as  a  decimal,  8%  =  .08) 
$pmt  =  first  or  re-occuring  payment  or  receipt  (this  number  cannot   
vary  for  differing  cash  flows) 
$principal  =  present  value  of  an  annuity,  or  current  principal  of  a   
loan  used  in  the  PaymentCalc  function 

*/ 

//  present  value  of  an  Annuity   
//  uses  include  defining  Remaining  Principal  of  a  loan/mortgage 

function  PVannuity  ($m,$n,$R,$pmt)  { 
         
        $Z  =  1  /  (1  +  ($R/$m)); 
        return  ($pmt  *  $Z  *  (1  -  pow($Z,$n)))/(1  -  $Z); 

} 

//  Given  the  compounding,  principal,  interest  rate,  you  can  calculate   
//  the  monthly  payment 

function  PaymentCalc  ($m,$n,$R,$principal)  { 

        $Z  =  1  /  (1  +  ($R/$m)); 
        return  ((1  -  $Z)  *  $principal)  /  ($Z  *  (1  -  pow($Z,$n))); 
         
} 

//  future  value  of  an  Annuity 

function  FVannuity  ($m,$n,$R,$pmt)  { 

        return  $pmt  *  ((pow((1  +  $R/$n),$m)  -  1)/($R/$n)); 

} 

//  present  value  of  a  single  payment 

function  PVsingle  ($m,$n,$R,$pmt)  { 

        return  $pmt  *  pow((1  +  $R/$m),-$n); 

} 

//  future  value  of  a  single  payment 

function  FVsingle  ($m,$n,$R,$pmt)  { 

        return  $pmt  *  pow((1  +  $R/$m),$n); 

} 
         
//  future  value  of  a  single  payment  with  continuous  compounding 

function  FVperp  ($m,$n,$R,$pmt)  { 

        return  $pmt  *  exp($R  *  ($n/$m)); 

} 

