<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This module will validate data based on the edits specified on the validation.xml file. 
 * There are two main functions here.  1) create JQuery validation routines for the 
 * jquery.validate plugin.  and 2) validate data on the server side.  Both functions
 * are driven from the same xml file so that validation between client and server
 * are kept in sync.  
 * 
 * This routine validates against the Post variables for validation against specified fields
 * see the routine sys_field_validator.php
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_validation {

	protected $_registry;
	protected $_request;
	protected $_config;

	// ---------------------------------------------------- constructor
	protected function __construct(){
		$this->_registry = sys_registry::getInstance();
		$this->_request = sys_request::getInstance();
		$this->_config = sys_config::getInstance();
	}

	public function createJqValidation($formName, $editGroup = null)  {
		if (!isset($editGroup)) {
			$editGroup = $formName;
		}
		$edit_dao = sys_edits::getInstance();
		$edits = $edit_dao->loadActiveEdits($editGroup);
//		var_dump($edits);
		if ($edits == null) {
			return '';
		}
		$fields = array();

		foreach ($edits as $edit) {
			$fieldName = $edit->getField1();
			if (!key_exists($fieldName,$fields)) {
				$field = new sys_edit_field_bean();
				$fields [$fieldName] = $field;
			}
			$field = $fields [$fieldName];
			$field->addEdit($edit);
		}
//		var_dump($fields);
		$result = "
 $().ready(function() {
    $('#$formName').validate({";
      $result .= $this->getAdditionalOptions();
      $result .= "
      rules: {\n";
		foreach ($fields as $field) {
			$fieldEdits = $field->getEdits();
			$result .= '        '.$fieldEdits[0]->getField1()." : {\n";
			foreach	 ($fieldEdits as $fieldEdit) {
				if       ($fieldEdit->getEditType() == 'required') {
					$result .= "            required: true";
				} elseif ($fieldEdit->getEditType() == 'num') {
					$result .= "      	    number: true";
				} elseif ($fieldEdit->getEditType() == 'a/n') {
					$result .= "      	    alphanumeric: true";
				} elseif ($fieldEdit->getEditType() == 'date') {
					$result .= "      	    date: true";
				} elseif ($fieldEdit->getEditType() == 'minlength') {
					$result .= "            minlength: ".$fieldEdit->getValue1();
				} elseif ($fieldEdit->getEditType() == 'maxlength') {
					$result .= '            maxlength: '.$fieldEdit->getValue1();
                } elseif ($fieldEdit->getEditType() == 'min') {
                    $result .= '            min: '.$fieldEdit->getValue1();
               } elseif ($fieldEdit->getEditType() == 'max') {
                    $result .= '            max: '.$fieldEdit->getValue1();
                } elseif ($fieldEdit->getEditType() == 'email') {
                    $result .= '            email: true';
                } elseif ($fieldEdit->getEditType() == 'equalTo') {
                    $result .= '            equalTo: "#'.$fieldEdit->getField2().'"';
                } else {
                	$result .= $this->getCustomJsEdit($fieldEdit);
                }
				$result .= ",\n";
			}
			$result = substr($result,0,-2)."\n";
			$result .= "	       },\n";
		}
		$result = substr($result,0,-2)."\n";
		$result .= "
	  },
	  messages: {\n";
		foreach ($fields as $field) {
			$fieldEdits = $field->getEdits();
			$result .= '        '.$fieldEdits[0]->getField1()." : {\n";
			foreach	 ($fieldEdits as $fieldEdit) {
				if       ($fieldEdit->getEditType() == 'required') {
					$result .= "            required: '".$fieldEdit->getEditText()."'";
				} elseif ($fieldEdit->getEditType() == 'num') {
					$result .= "      	    number: '".$fieldEdit->getEditText()."'";
				} elseif ($fieldEdit->getEditType() == 'a/n') {
					$result .= "      	    alphanumeric: '".$fieldEdit->getEditText()."'";
				} elseif ($fieldEdit->getEditType() == 'date') {
					$result .= "      	    date: '".$fieldEdit->getEditText()."'";
				} elseif ($fieldEdit->getEditType() == 'minlength') {
					$result .= "            minlength: '".$fieldEdit->getEditText()."'";
				} elseif ($fieldEdit->getEditType() == 'maxlength') {
					$result .= "            maxlength: '".$fieldEdit->getEditText()."'";
                } elseif ($fieldEdit->getEditType() == 'min') {
                    $result .= "            min:  '".$fieldEdit->getEditText()."'";
               } elseif ($fieldEdit->getEditType() == 'max') {
                    $result .= "            max:  '".$fieldEdit->getEditText()."'";
                } elseif ($fieldEdit->getEditType() == 'email') {
                    $result .= "            email:  '".$fieldEdit->getEditText()."'";
                } elseif ($fieldEdit->getEditType() == 'equalTo') {
                    $result .= "            equalTo:  '".$fieldEdit->getEditText()."'";
                } else {
                	$result .= $this->getCustomEditText($fieldEdit);
                }
				$result .= ",\n";
			}
			$result = substr($result,0,-2)."\n";
			$result .= "	       },\n";
		}	
		$result = substr($result,0,-2)."\n";
		$result .= '},
  });
});';
		return $result;
	}

	//  override in decendents to add custom edits.
	public function getCustomJsEdit($fieldEdit) {
		return '';
	}

	//  override in decendents to add custom edit text.
	public function getCustomEditText($fieldEdit) {
		return '';
	}
	
    public function getAdditionalOptions() {
    	return "";
    }
    
	public function ValidData($editGroup, $feedback)  {
		$dataIsValid = true;
		$edit_dao = sys_edits::getInstance();
		$edits = $edit_dao->loadActiveEdits($editGroup);
		if ($edits == null) {
			return $dataIsValid;
		}
		foreach ($edits as $edit) {
			if       ($edit->getEditType() == 'required') {
				$result = $this->EditReq($edit, $feedback);
			} elseif ($edit->getEditType() == 'num') {
				$result = $this->EditNum($edit, $feedback);
			} elseif ($edit->getEditType() == 'a/n') {
				$result = $this->EditAN($edit, $feedback);
			} elseif ($edit->getEditType() == 'date') {
				$result = $this->EditDate($edit, $feedback);
            } elseif ($edit->getEditType() == 'minlength') {
                $result = $this->EditMinLen($edit, $feedback);
            } elseif ($edit->getEditType() == 'maxlength') {
                $result = $this->EditMaxLen($edit, $feedback);
            } elseif ($edit->getEditType() == 'min') {
                $result = $this->EditMin($edit, $feedback);
            } elseif ($edit->getEditType() == 'max') {
                $result = $this->EditMax($edit, $feedback);
            } elseif ($edit->getEditType() == 'email') {
                $result = $this->EditEmail($edit, $feedback);
            } elseif ($edit->getEditType() == 'equealTo') {
                $result = $this->EditEqualTo($edit, $feedback);
            } else {
            	$result = $this->getCustomServerEdit($edit, $feedback);
            }
			if ($result == false)
			$dataIsValid = false;
		}
		return $dataIsValid;
	}
	

	//  override in decendents to add custom edits.
	public function getCustomServerEdit($edit, $feedback) {
		return true;
	}
	
	
	function EditReq($edit, $feedback)  {
		$dataIsValid = true;
		$field1  = $edit->getField1();
		$var     = $this->_request->getPostVar($field1);
		if ($var == null) {
			$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		} else
		if (!$dataIsValid || trim($var) == '')  {
			$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}

	function EditAN($edit, $feedback) {
		$dataIsValid = true;
		$field1  = $edit->getField1();
		$var     = $this->_request->getPostVar($field1,'string');
		if ($var == null) {
			return true;
		}
		if (trim($var) == '')  {
			return true;
		}
		if (!ctype_alnum($var)) {
			$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}

	function EditNum($edit, $feedback) {
		$dataIsValid = true;
		$field1  = $edit->getField1();
		$var     = $this->_request->getPostVar($field1,'string');
		if ($var == null) {
			return true;
		}
		if (trim($var) == '')  {
			return true;
		}
		if (!is_numeric($var)) {
			$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}

	function EditDate($edit, $feedback)
	{
		$dataIsValid = true;
		$field1  = $edit->getField1();
		$var     = $this->_request->getPostVar($field1,'string');
		if ($var == null) {
			return true;
		}
		if (trim($var) == '')  {
			return true;
		}
		if (is_numeric($var) && strlen($var) == 6) {
			$m = substr($var,0,2);
			$d = substr($var,2,2);
			$y = substr($var,4,2);
			if ($y > 30) {
				$y += 1900;
			} else {
				$y += 2000;
			}
		} elseif (is_numeric($var) && strlen($var) == 8) {
			$m = substr($var,0,2);
			$d = substr($var,2,2);
			$y = substr($var,4,4);
		} elseif (strpos($var,'/') != false) {
			$date_array = explode('/',$var);
			$m = $date_array[0];
			$d = $date_array[1];
			$y = $date_array[2];
			if (strlen($y) == 2) {
				if ($y > 30) {
					$y += 1900;
				} else {
					$y += 2000;
				}
			}
		} elseif (strpos($var,'-') != false) {
			$date_array = explode('-',$var);
			$y = $date_array[0];
			$m = $date_array[1];
			$d = $date_array[2];
		}


		if (!checkdate($m,$d,$y)) {
			$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		} else {
			$var = mktime(0,0,0,$m,$d,$y);
			$var = date('Y-m-d',$var);
			$this->_request->setPostVar($field1,$var);
		}
		return $dataIsValid;
	}

    function EditMaxLen($edit, $feedback)  {
        $dataIsValid = true;
        $field1  = $edit->getField1();
        $var     = $this->_request->getPostVar($field1,'string');
        if ($var == null) {
            return true;
        }
        if (trim($var) == '')  {
            return true;
        }
        $lengthFactor = strlen(trim($var));
        $maxFactor = $edit->getValue1();
        if ($lengthFactor > $maxFactor) {
            $editText = $edit->getEditText();
            $feedback->setErr($editText);
            $dataIsValid = false;
        }
        return $dataIsValid;
    }

    function EditMax($edit, $feedback)  {
        $dataIsValid = true;
        $field1  = $edit->getField1();
        $var     = $this->_request->getPostVar($field1,'string');
        if ($var == null) {
            return true;
        }
        if (trim($var) == '')  {
            return true;
        }
        if (trim($var) > $edit->getValue1()) {
            $editText = $edit->getEditText();
            $feedback->setErr($editText);
            $dataIsValid = false;
        }
        return $dataIsValid;
    }
	
    function EditMinLen($edit, $feedback)  {
        $dataIsValid = true;
        $field1  = $edit->getField1();
        $var     = $this->_request->getPostVar($field1,'string');
        if ($var == null) {
            return true;
        }
        if (trim($var) == '')  {
            return true;
        }
        $lengthFactor = strlen(trim($var));
        $minFactor = $edit->getValue1();
        if ($lengthFactor < $minFactor) {
            $editText = $edit->getEditText();
            $feedback->setErr($editText);
            $dataIsValid = false;
        }
        return $dataIsValid;
    }

    function EditMin($edit, $feedback)  {
        $dataIsValid = true;
        $field1  = $edit->getField1();
        $var     = $this->_request->getPostVar($field1,'string');
        if ($var == null) {
            return true;
        }
        if (trim($var) == '')  {
            return true;
        }
        if (trim($var) < $edit->getValue1()) {
            $editText = $edit->getEditText();
            $feedback->setErr($editText);
            $dataIsValid = false;
        }
        return $dataIsValid;
    }
    
	function EditEmail($edit, $feedback) {
		$dataIsValid = true;
		$field1  = $edit->getField1();
		$var     = $this->_request->getPostVar($field1,'string');
		if ($var == null) {
			return true;
		}
		if (trim($var) == '')  {
			return true;
		}
//        if(!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $var)) {
        if (!preg_match("/^[[:alnum:]][a-z0-9_.-]*@[a-z0-9.-]+\.[a-z]{2,4}$/i", $var)) {
        	$editText = $edit->getEditText();
			$feedback->setErr($editText);
			$dataIsValid = false;
		}
		return $dataIsValid;
	}

    function EditEqualTo($edit, $feedback)  {
        $dataIsValid = true;
        $field1  = $edit->getField1();
        $field2  = $edit->getField2();
        $var1    = $this->_request->getPostVar($field1,'string');
        $var2    = $this->_request->getPostVar($field2,'string');
        if ($var1== null) {
            return true;
        }
        if (trim($var1) == '')  {
            return true;
        }
        if ($var1 != $var2) {
            $editText = $edit->getEditText();
            $feedback->setErr($editText);
            $dataIsValid = false;
        }
        return $dataIsValid;
    }
	
}