<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * This is the base class for navigation actions for the Hard Hat Framework
 * 
 * This class is meant to be extended in the application to handle navigation action requests.  Navigation
 * actions handle http get requests.
 *
 * @package     hardhat_framework
 * @subpackage  controller
 * @author      Rod Johnson
 * 
 */
class sys_navigation_action
{
	protected $_request;
	protected $_registry;
	protected $_config;
	protected $_admin;
	protected $_codes;
	protected $_userName;
	protected $_loggedIn;

	// ---------------------------------------------------- constructor
	public function __construct(){
		$this->_request   = sys_request::getInstance();
		$this->_registry = sys_registry::getInstance();
		$this->_config   = sys_config::getInstance();
	}
	// ---------------------------------------------------- perform
	public function perform($action,$feedback,&$login){
		$template = $action->getTemplate();
		$cmd = $action->getPath();
		$msg = $feedback->getMsg();
		$errors = $feedback->getErrors();
		$smarty = new Smarty;
		$smarty->compile_check = true;
		$smarty->debugging = false;
		$smarty->setTemplateDir(SMARTY_TEMPLATE_DIR);
		$smarty->setCompileDir(SMARTY_COMPILE_DIR);
		$smarty->setCacheDir(SMARTY_CACHE_DIR);
		$smarty->setConfigDir(SMARTY_CONFIG_DIR);
		$loggedIn = $this->_registry->getLoggedIn();
		$this->_admin = $login->getAdmin();
		$userName = $login->getUserName();
		$className = get_class($this);
		$this->_userName = $login->getUserName();
		$this->_loggedIn = $loggedIn;
		$config = $this->_config->getConfig();
		foreach ($config as $key => $val) {
			if ($key != 'password') {
				$smarty->assign($key, $val);
			}
		}
		$smarty->assign('loggedIn', $loggedIn);
		$smarty->assign('userName', $login->getUserName());
		$smarty->assign('admin', $this->_admin);
		$smarty->assign("feedback", $feedback);
		$smarty->assign("cmd",$cmd);
		$smarty->assign("className",$className);
		$smarty->assign("template",$template);
		$smarty->assign("action",$action);
		$smarty->assign("application_path", APPLICATION_PATH);
		$this->prepareTemplate($action,$feedback,$smarty,$login);
		try {
			return $this->displayTemplate ($action,$feedback,$smarty,$login);
		}
		
		catch (exception $e) {
			$feedback->setMessage('Navigation Error', $e->getMessage());
//          $feedback->setMsg('Something is wrong, possibly broken. Please let us know if you are having trouble.  We would very much like to fix any problems.', 'E');
            $debug = "navigation error: \n".$e->getMessage();
            $debug .= " \n".callStack($e->getTrace());
            $feedback->setDebug($debug);
            $feedback->setOriginalUrl($_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
			@$feedback->setReferrer ( $_SERVER ['HTTP_REFERER'] );
            $this->_registry->setEntry('core','feedback', $feedback);
        	$urlString = makeRelativeURL('error');
		    header("Location: $urlString");
			exit;
		}
	}
	
	public function getLoggedIn() {
		return $this->_loggedIn;
	}
	public function displayTemplate($action,$feedback,$smarty,&$login){
		$template = $action->getTemplate();
		$smarty->display($template);
		return 'done';
	}
	public function prepareTemplate($action,$feedback,$smarty,&$login){
		return;
	}



}
