<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This is the base class for post actions for the Hard Hat Framework
 *
 * This class is meant to be extended in the application to handle post action requests.  Post
 * actions handle http post requests.  When the post action is complete it returns a "cmd" which
 * is really navigation action or http get request, which will display a screen in response.
 * 
 *
 * @package     hardhat_framework
 * @subpackage  controller
 * @author      Rod Johnson
 * 
 */
class sys_post_action {

protected $_registry;
protected $_request;	
protected $_config;	
protected $_submit;

// ---------------------------------------------------- constructor
protected function __construct(){
	$this->_registry = sys_registry::getInstance();
    $this->_request = sys_request::getInstance();
    $this->_config = sys_config::getInstance();
}
public function process($action,$feedback,&$login) {
    throw new Exception('please override the process method');
}

public function applyPostVars($bean,$checkboxes = array(),$foundVal = 'Y', $notFoundVal = 'N') {
    if (!is_subclass_of($bean,'sys_base_bean')) {
        throw new Exception('applyPostVars 1st argument must be a bean');
    }
    $fields = $bean->getBeanFields();
    foreach ($fields as $key => $val) {
        if (in_array($key,$checkboxes)) {
           if ($this->_request->hasPostVar($key)) {
               $fields [$key] = $foundVal;
           } else {
               $fields [$key] = $notFoundVal;
           }
        } elseif ($this->_request->hasPostVar($key)) {
            $fields [$key] = $this->_request->getPostVar($key);
        }
    }
    $bean->setBeanFields($fields);
    return $bean;    
}

}