<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This is the base class for dao's in the Hard Hat Framework
 *
 * This class is meant to be extended in the application to database access.  The constructor specifies
 * a table, view, and a bean.  The view is used for reading data and returns a bean or array of beans.
 * The table is used for insert, update and delete processing.
 * 
 *
 * @package     hardhat_framework
 * @subpackage  controller
 * @author      Rod Johnson
 * 
 */
class sys_base_dao
{
    protected $_conn;
    private $_tableName;
    private $_viewName;
    private $_beanName;
    private $_schema;
    private $_viewSchema;
    private $_fieldList;  
    private $_viewFieldList;
    private $_daoFields;     
    private $_viewDaoFields;
    private $_identity;

    // ---------------------------------------------------- constructor
    public function __construct($tableName, $beanName, $viewName = null, $identity = 'id'){
        if ($viewName == null) {
            $viewName = $tableName;
        }
        $this->_identity = $identity;
        $connObject = sys_connect_db::getInstance();
        $this->_conn = $connObject->getConnection();
        $this->_viewSchema = array();
        $this->_viewFieldList = '';
        $this->_viewDaoFields = array();
        
/*
 *      load the view fieldList for select processing 
 */        
        $query  = "SHOW COLUMNS FROM $viewName";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $fieldInfo = $row;
            $type = $row['Type'];
            $dataType = snip($type,'(');
            $length = snip($type,')');
            $fieldInfo ['DataType'] = $dataType;
            $fieldInfo ['Length'] = $length;
            $fieldName = strtolower($row['Field']);
            $this->_viewSchema [$fieldName] = $fieldInfo;
            $this->_viewDaoFields [] = strtolower($row['Field']);
            $this->_viewFieldList .= strtolower($row['Field']).', ';
        }
        $this->_viewFieldList = substr($this->_viewFieldList,0,-2);

/*
 *   load the table daoFields for insert update processing
 */
        $this->_schema = array();
        $this->_fieldList = '';
        $this->_daoFields = array();
        $query  = "SHOW COLUMNS FROM $tableName";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $fieldInfo = $row;
            $type = $row['Type'];
            $dataType = snip($type,'(');
            $length = snip($type,')');
            $fieldInfo ['DataType'] = $dataType;
            $fieldInfo ['Length'] = $length;
            $fieldName = strtolower($row['Field']);
            $this->_daoFields [] = strtolower($row['Field']);
            $this->_schema [$fieldName] = $fieldInfo;
            $this->_viewDaoFields [] = strtolower($row['Field']);
            $this->_fieldList .= strtolower($row['Field']).', ';
        }
        $this->_fieldList = substr($this->_fieldList,0,-2);
        $this->_beanName = $beanName;
        $this->_tableName = $tableName;
        $this->_viewName = $viewName;
    }

    // ---------------------------------------------------- getters
    public function getEmptyBean(){
        $beanName = $this->getBeanName();
        $bean = new $beanName;
        $fields = $this->getSchema();
        foreach ($fields as $key => $val) {
            $bean->set($key,'');
        }
        return $bean;
   }
    public function getTableName(){
        return $this->_tableName;
    }
    public function getViewName(){
        return $this->_viewName;
    }
    public function getBeanName(){
        return $this->_beanName;
    }
    public function getSchema(){
        return $this->_schema;
    }
    public function getViewSchema(){
        return $this->_viewSchema;
    }
    public function getFieldList(){
        return $this->_fieldList;
    }
    public function getViewFieldList(){
        return $this->_viewFieldList;
    }
    public function getDaoFields(){
        return $this->_daoFields;
    }
    public function getViewDaoFields(){
        return $this->_viewDaoFields;
    }
    public function getFieldInfo($fieldName) {
        $tableName = $this->_tableName;
        $fields = $this->getViewDaoFields();
        if (!in_array($fieldName, $fields)) {
            $className = get_class($this);
            throw new Exception("Class $className - field ($fieldName) not found in table ($tableName).");
            exit();
        }
        $schema = $this->getViewSchema();
        return $schema [$fieldName];
    }

    private function getBean($row){

        $beanName = $this->getBeanName();
        $bean = new $beanName;
        foreach ($row as $field => $value) {
        	$field = strtolower($field);
            $fieldInfo = $this->getFieldInfo($field);
            $dataType = $fieldInfo ['DataType'];
            if ($dataType == 'date') {
                $value = strtodate($value);
            }
            $setMethod = "set$field";
            $bean->$setMethod($value);
        }
        $bean = $this->buildBean($bean);
        return $bean;
    }

    protected function buildBean($bean) {
        return $bean;
    }

    public function prep($data, $field){
        //     return fmtString(serialize($data));
        $fieldInfo = $this->getFieldInfo($field);
        $dataType = $fieldInfo ['DataType'];
        $dataLength = $fieldInfo ['Length'];
        if (is_object($data)) {
            return fmtString(serialize($data));
        } else {
            if (is_array($data)) {
                return fmtString(serialize($data));
            }
            switch ($dataType) {
                case 'int'      : return fmtInt($data);
                case 'text'     :
                case 'longtext' :	
                case 'char'     :
                case 'enum'     :
                    //           case 'varchar'  : return fmtString(substr($data,1,$dataLength));
                case 'varchar'  : return fmtString($data);
                case 'decimal'  : return $data;
                case 'date'     : if (isset($data) && trim($data) != '') {
                    return fmtDbDate($data);
                } else {
                    return 'null';
                }
                case 'datetime' :  if (strpos($data,'T') > 0) {
                    return "$data";
                } else {
                    return "'$data'";
                }
                default         : //echo $dataType;
                    return fmtString(serialize($data));
            }
        }
        //*/
    }
    protected function runQuery($query){
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        if ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            return $row;
        }
        return null;
    }
     

   public function executeQuery($query){
      $lineNbr = __LINE__ + 1;
      $result = mysql_query($query,$this->_conn);

      if (!$result) {
         $dbError = new sys_db_error_log_bean();
         $fileName = __FILE__;
         $errorString = mysql_error();
         $dbError->init($query, $fileName, $lineNbr, $errorString);
         $dbError->report();
         exit;
      }
      return $result;
   }
    
    protected function getResult($query) {
        $array = array();
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        $num_results = mysql_num_rows($result);
        for ($i = 0; $i < $num_results; $i++) {
            if ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
                $array [] = $row;
            }
        }
        return $array;
    }

    protected function getFlags($fieldName){
        $fieldInfo = $this->getFieldInfo($fieldName);
        $tableName = $this->getTableName();
        $beans = array();
        $query  = "select distinct $fieldName from $tableName";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $bean = new sys_flag_bean();
            $bean->set('name',$row[$fieldName]);
            $beans [] = $bean;
        }
        return $beans;
    }

    protected function loadBeanById($id){
        $beanName = $this->getBeanName();
        $bean = new $beanName;
        $flist = $this->getFieldList();
        $tableName = $this->getViewName();
        $id = fmtInt($id);
        $query  = "select $flist from $tableName where ";
        $query .= $this->_identity." = $id";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        if ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $bean = $this->getBean($row);
        }
        return $bean;
    }

    protected function loadBean($criteria, $orderBy = null){
        // $beanName = $this->getBeanName();
        // $bean = new $beanName;
        $bean = null;
        $flist = $this->getViewFieldList();
        $tableName = $this->getViewName();
        $query  = "select $flist from $tableName where ";
        $query .= "$criteria ";
        if ($orderBy != null) {
            $query .= "order by $orderBy";
        }
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        if ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $bean = $this->getBean($row);
        }
        return $bean;
    }
    protected function loadBeans($criteria, $orderBy = null, $limit = null){
        $flist = $this->getViewFieldList();
        $tableName = $this->getViewName();
        $query  = "select $flist from $tableName where ";
        $query .= "$criteria ";
        if ($orderBy != null) {
            $query .= " order by $orderBy ";
        }
        if ($limit != null) {
            $query .= " Limit $limit ";
        }
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        $beans = array();
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
            $bean = $this->getBean($row);
            $beans [] = $bean;
        }
        return $beans;
    }
    protected function saveBean($bean,$user){
        $id = $bean->getId();
        if (!isset($id)) {
            return $this->insertBean($bean,$user);
        }
        if ($id == 0) {
            return $this->insertBean($bean,$user);
        }
        $temp = $this->loadBeanById($bean->getId());
        if ($temp == null) {
            return $this->insertBean($bean,$user);
        } else {
            return $this->updateBean($bean,$user);
        }
    }
     
    protected function insertBean(sys_base_bean $bean, $user = null){
        if ($user == null) {
            $user = new sys_user_bean();
        }
        $fields = $this->getDaoFields();
        $tableName = $this->getTableName();
        $flist = '';
        $values = '';
        foreach ($fields as $field) {
            $lowerfield = strtolower($field);
            if ($bean->fieldExists($lowerfield) && $lowerfield != strtolower($this->_identity) &&
            $lowerfield != 'updatedate' &&
            $lowerfield != 'createdate' &&
            $lowerfield != 'createby' &&
            $lowerfield != 'updateby') {
                $flist .= $field.", ";
                $fieldInfo = $this->getFieldInfo($lowerfield);
                $dataType = $fieldInfo ['DataType'];
                $getMethod = "get$lowerfield";
                $value = $bean->$getMethod();
                $value = $this->prep($value,$lowerfield);
                $values .= "$value, ";
            } else {
                $found = false;
                switch (strtolower($field)) {
                    case 'updatedate' :
                    case 'createdate' : $value = fmtCurrentDate();
                    $found = true;
                    break;
                    case 'createby'   :
                    case 'updateby'   : $value = fmtIntDefault($user->getId(),0);
                    $found = true;
                    break;
                }
                if ($found) {
                    $flist .= strtolower($field).", ";
                    $values .= "$value, ";
                } else {
//                	echo 'not found '.$field.' '.$value;
                	
                }
            }
        }
        $flist = substr($flist,0,-2);
        $values = substr($values,0,-2);
        $query  = "INSERT INTO $tableName ($flist) values ($values)";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        return mysql_insert_id();
    }
    protected function updateBean(sys_base_bean $bean, sys_user_bean $user){
        $fields = $this->getDaoFields();
        $tableName = $this->getTableName();
        $flist = $this->getFieldList();
        $query  = "UPDATE $tableName SET ";
        foreach ($fields as $field) {
        	$field = strtolower($field);
            if (($field != $this->_identity) && $bean->fieldExists($field)) {
                $fieldInfo = $this->getFieldInfo($field);
                $dataType = $fieldInfo ['DataType'];
                $getMethod = "get$field";
                $value = $bean->$getMethod();
                switch (strtolower($field)) {
                    case 'updatedate' : $value = fmtCurrentDate();
                    break;
                    case 'updateby'   : $value = fmtIntDefault($user->getId(),0);
                    break;
                    default :
                        $value = $this->prep($value,$field);
                }
                $query .= "$field = $value, ";
            }
        }
        $query = substr($query,0,-2);
        $value = fmtInt($bean->getId());
        $query .= " where $this->_identity = $value";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        $cnt = mysql_affected_rows();
        return $cnt;
    }

    protected function deleteBeans($criteria){
        $tableName = $this->getTableName();
        $query  = "DELETE from $tableName where $criteria";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        $cnt = mysql_affected_rows();
        return $cnt;
    }

    protected function deleteBean($id){
        $tableName = $this->getTableName();
        $query  = "DELETE from $tableName where $this->_identity = $id";
        $lineNbr = __LINE__ + 1;
        $result = mysql_query($query,$this->_conn);
        if (trim(mysql_error()) != '') {
            $dbError = new sys_db_error_log_bean();
            $fileName = __FILE__;
            $errorString = mysql_error();
            $dbError->init($query, $fileName, $lineNbr, $errorString);
            $dbError->report();
            exit;
        }
        $cnt = mysql_affected_rows();
        return $cnt;
    }

}
