<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

class sys_criteria_bean extends sys_base_bean
{
   private $_fields;
   private $_cd;

   // ---------------------------------------------------- constructor
   public function __construct(){
      parent::__construct();
      $this->_cd = sys_codes::getInstance();
      $this->_fields = array();
   }

   public function addField($fieldName, $codeName, $codeType) {
      $criterion = $this->_cd->getCriterion($fieldName, $codeName, $codeType);
      $this->_fields [$fieldName] = $criterion;
   }

   //   public function getFieldFlags($fieldName) {
   //      return array_key_exists($fieldName,$this->_fields) ? $this->_fields [$fieldName] : array();
   //   }

   public function getField($fieldName) {
      return array_key_exists($fieldName,$this->_fields) ? $this->_fields [$fieldName] : array();
   }

   public function getCriteriaSql($fieldName) {
      $field = $this->getField($fieldName);
      $flags = $field->getFlags();
      $codeType = $field->getCodeType();
      $sql = '(';
      foreach ($flags as $flag) {
         $code = $flag->getName();
         if ($flag->getFieldValue() == 'Y'){
            $sql .= " $fieldName = ";
            if ($codeType == 'C') {
               $sql .= " '$code' or";
            } else {
               $sql .= " $code or";
            }
         }
      }
      if (substr($sql, -2) == 'or') { //strip the last or
         $sql = substr($sql, 0, -2);
      }
      $sql .= ")";
      if ($sql == "()") return " 1=1 ";
      return $sql;
       
   }

   public function getFields() {
      return $this->_fields;
   }

   public function getFieldMaxWidth() {
      $maxWidth = 0;
      foreach ($this->_fields as $criterion) {
         if (count($criterion->getFlags()) > $maxWidth) {
            $maxWidth = count($criterion->getFlags());
         }
      }
      return $maxWidth;
   }

   public function getFieldWidth($fieldName) {
      return array_key_exists($fieldName,$this->_fields) ? count($this->_fields[$fieldName]->getFlags()) : 0;
   }

}