<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

class sys_db_error_log_bean extends sys_base_bean {

	protected $_request;
	protected $_registry;
	protected $_config;
	protected $_dao;

	// ---------------------------------------------------- constructor
	public function __construct(){
		parent::__construct();
		$this->_request = sys_request::getInstance();
		$this->_registry = sys_registry::getInstance();
		$this->_config   = sys_config::getInstance();
	}
	 

	public function init($queryString, $fileName, $lineNbr, $sqlErrorText){
		$this->_dao      = new sys_db_error_dao();
		$login = $this->_registry->getEntry('core', 'login');
		$this->setLogTimeStamp(strtotime('now'));
		if (is_object($login)) {
			$this->setLoginId     ($login->getId());
			$this->setUserId      ($login->getId());
			$this->setUserName    ($login->getFullName());
		}
		$this->setPostVars    (serialize($this->_request->getPostVars()));
		$this->setGetVars     (serialize($this->_request->getUrlFields()));
		$this->setQueryString ($queryString);
		$this->setFileName    ($fileName);
		$this->setLineNbr     ($lineNbr);
		$this->setSqlErrorText($sqlErrorText);
	}

	public function getFormattedMessage() {
		$ts = $this->getLogTimeStamp();
		$un = $this->getUserName();
		$qs = $this->getQueryString();
		$fn = basename($this->getFileName());
		$ln = $this->getLineNbr();
		$et = $this->getSqlErrorText();
		$s  = "<table border=\"1\">";
		$s .= "<tr><td>";
		$s .= "DateTime:</td><td>$ts\n\r";
		$s .= "</td></tr>";
		$s .= "<tr><td>";
		$s .= "User:</td><td>$un \n\r";
		$s .= "</td></tr>";
		$s .= "<tr><td>";
		$s .= "Query:</td><td>$qs \n\r";
		$s .= "</td></tr>";
		$s .= "<tr><td>";
		$s .= "file/line:</td><td>$fn:$ln \n\r";
		$s .= "</td></tr>";
		$s .= "<tr><td>";
		$s .= "SQL Error:</td><td>$et</td></tr>\n\r";
		$s .= "</table>";
		return $s;
	}

	public function report() {
		if ($this->_config->getEntry('showErrors') == 'Y') {
			echo $this->getFormattedMessage();
			echo '<pre>';
			debug_print_backtrace();
		}
		$this->_dao->insertLog($this,$this->_registry->getEntry('core', 'login'));
		$un = $this->getUserName();
		$qs = $this->getQueryString();
		$fn = basename($this->getFileName());
		$ln = $this->getLineNbr();
		$et = $this->getSqlErrorText();
		error_log("User:$un - query string:$qs - filename:$fn - line:$ln - error:$et");
	}

	public function reportNoDb() {
		if ($this->_config->getEntry('showErrors' == 'Y')) {
			echo $this->getFormattedMessage();
		}
	}

}
